@@ -15,7 +15,6 @@
  */
 
 #include "kerncompat.h"
-#include "androidcompat.h"
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -156,7 +155,7 @@ static int fill_inode_item(struct btrfs_trans_handle *trans,
 	return 0;
 }
 
-static int directory_select(const struct direct *entry)
+static int directory_select(const struct dirent *entry)
 {
 	if (entry->d_name[0] == '.' &&
 		(entry->d_name[1] == 0 ||
@@ -165,7 +164,7 @@ static int directory_select(const struct direct *entry)
 	return 1;
 }
 
-static void free_namelist(struct direct **files, int count)
+static void free_namelist(struct dirent **files, int count)
 {
 	int i;
 
@@ -180,7 +179,7 @@ static void free_namelist(struct direct **files, int count)
 static u64 calculate_dir_inode_size(const char *dirname)
 {
 	int count, i;
-	struct direct **files, *cur_file;
+	struct dirent **files, *cur_file;
 	u64 dir_inode_size = 0;
 
 	count = scandir(dirname, &files, directory_select, NULL);
@@ -445,10 +444,10 @@ static int traverse_directory(struct btrfs_trans_handle *trans,
 	struct btrfs_inode_item cur_inode;
 	struct btrfs_inode_item *inode_item;
 	int count, i, dir_index_cnt;
-	struct direct **files;
+	struct dirent **files;
 	struct stat st;
 	struct directory_name_entry *dir_entry, *parent_dir_entry;
-	struct direct *cur_file;
+	struct dirent *cur_file;
 	ino_t parent_inum, cur_inum;
 	ino_t highest_inum = 0;
 	const char *parent_dir_name;
